@@ -1,12 +1,13 @@
 #
 # linearK
 #
-# $Revision: 1.63 $ $Date: 2023/02/20 03:41:27 $
+# $Revision: 1.66 $ $Date: 2025/09/03 04:08:52 $
 #
 # K function for point pattern on linear network
 #
 #
 linearK <- function(X, r=NULL, ..., correction="Ang", ratio=FALSE) {
+  if(is.NAobject(X)) return(NAobject("fv"))
   stopifnot(inherits(X, "lpp"))
   correction <- pickoption("correction", correction,
                            c(none="none",
@@ -38,8 +39,9 @@ linearK <- function(X, r=NULL, ..., correction="Ang", ratio=FALSE) {
 
 linearKinhom <- function(X, lambda=NULL, r=NULL,  ...,
                          correction="Ang", normalise=TRUE, normpower=1,
-			 update=TRUE, leaveoneout=TRUE,
+			 update=TRUE, leaveoneout=TRUE, sigma=NULL,
 			 ratio=FALSE) {
+  if(is.NAobject(X)) return(NAobject("fv"))
   stopifnot(inherits(X, "lpp"))
   loo.given <- !missing(leaveoneout)
   correction <- pickoption("correction", correction,
@@ -47,8 +49,13 @@ linearKinhom <- function(X, lambda=NULL, r=NULL,  ...,
                              Ang="Ang",
                              best="Ang"),
                            multi=FALSE)
-  if(is.null(lambda))
-    linearK(X, r=r, ..., ratio=ratio, correction=correction)
+  if(is.null(lambda)) 
+    warn.once("linearKinhomNULL",
+              "In linearKinhom the interpretation of 'lambda=NULL'",
+              "has changed (in spatstat.linnet 3.1 and later);",
+              "the function linearK is no longer invoked;",
+              "instead the intensity lambda is estimated by kernel smoothing")
+
   if(normalise) {
     check.1.real(normpower)
     stopifnot(normpower >= 1)
@@ -56,6 +63,7 @@ linearKinhom <- function(X, lambda=NULL, r=NULL,  ...,
   lambdaX <- resolve.lambda.lpp(X, lambda, ...,
                                 update=update, leaveoneout=leaveoneout,
                                 loo.given=loo.given,
+                                sigma=sigma,
                                 lambdaname="lambda")
   invlam <- 1/lambdaX
   invlam2 <- outer(invlam, invlam, "*")
@@ -93,11 +101,23 @@ resolve.lambda.lpp <- function(X, lambda, subset=NULL, ...,
                                update=TRUE, leaveoneout=TRUE,
                                everywhere=FALSE,
                                loo.given=TRUE,
+                               sigma=NULL,
                                lambdaname) {
-  if(missing(lambdaname)) lambdaname <- deparse(substitute(lambda))
+  if(missing(lambdaname)) lambdaname <- short.deparse(substitute(lambda))
   Y <- if(is.null(subset)) X else X[subset]
   danger <- TRUE
-  if(is.ppm(lambda) || is.lppm(lambda)) {
+  if(is.null(lambda)) {
+    ## compute a kernel density estimate at X
+    lambdaX <- density(X, sigma=sigma, ..., at="points",
+                       leaveoneout=leaveoneout)
+    ## ensure it is positive
+    lambdaX <- pmax(lambdaX, .Machine$double.eps)
+    ## Restrict if required
+    lambdaY <- if(is.null(subset)) lambdaX else lambdaX[subset]
+    ## Compute at all locations if required
+    if(everywhere)
+      everylambda <- density(X, sigma=sigma, ..., at="pixels") 
+  } else if(is.ppm(lambda) || is.lppm(lambda)) {
     ## fitted model
     model <- lambda
     if(update) {
@@ -156,6 +176,7 @@ resolve.lambda.lpp <- function(X, lambda, subset=NULL, ...,
 linearKengine <- function(X, ..., r=NULL, reweight=NULL,
                           denom=1, samplesize=NULL,
                           correction="Ang", ratio=FALSE, showworking=FALSE) {
+  if(is.NAobject(X)) return(NAobject("fv"))
   # ensure distance information is present
   X <- as.lpp(X, sparse=FALSE)
   # extract info about pattern
